# -*- coding: utf-8 -*-

import json
import os
import tarfile

from . import repo_local
from . import repo_publish
from .config import PUB_APPS_DIR
from .container import Container

class App:
    def __init__(self, name, load_from_repo=True):
        self.name = name
        self.version = None
        self.meta = {}
        self.containers = []
        if load_from_repo:
            self.set_definition(repo_local.get_app(name))

    def set_definition(self, definition):
        self.version = definition['version']
        self.meta = definition['meta']
        self.containers = [Container(container) for container in definition['containers']]

    def get_definition(self):
        return {
            'version': self.version,
            'meta': self.meta.copy(),
            'containers': [container.name for container in self.containers]
        }

    def publish(self, filename):
        builddir = os.path.dirname(filename)
        os.makedirs(PUB_APPS_DIR, 0o700, True)
        files = repo_publish.TarSizeCounter()
        archive_path = os.path.join(PUB_APPS_DIR, f'{self.name}.tar.xz')
        with tarfile.open(archive_path, 'w:xz') as tar:
            for content in ('install', 'install.sh', 'update', 'update.sh', 'uninstall', 'uninstall.sh'):
                content_path = os.path.join(builddir, content)
                if os.path.exists(content_path):
                    tar.add(content_path, os.path.join(self.name, content), filter=files.add_file)
        with open(filename) as f:
            definition = json.load(f)
        definition['size'] = files.size
        definition['dlsize'] = os.path.getsize(archive_path)
        definition['hash'] = repo_publish.sign_file(archive_path).hex()
        repo_publish.register_app(self.name, definition)
        return (definition['size'], definition['dlsize'])

    def unpublish(self):
        repo_publish.unregister_app(self.name)
        archive_path = os.path.join(PUB_APPS_DIR, f'{self.name}.tar.xz')
        try:
            os.unlink(archive_path)
        except FileNotFoundError:
            pass
